library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity system is
  port (
    clk : in std_logic;           -- external clock
    
    PB_D0, PB_D1, PB_D2, PB_D3, PB_D4, PB_D5, PB_D6, PB_D7 : out std_logic;
    
    PB_D8, PB_D9, PB_D10, PB_D11,
    PB_D12, PB_D13, PB_D14, PB_D15 : out std_logic
    );  
end system;

architecture imp of system is

  component hex2led
    port (
      hex : in std_logic_vector(3 downto 0);
      led : out std_logic_vector(6 downto 0) );
  end component;

  signal cnt24 : std_logic_vector(23 downto 0) := X"000000";  -- clk divider
  signal cnt8 : std_logic_vector(7 downto 0) := X"00";        -- our counter
  signal slow_clk : std_logic;

  signal leftleds  : std_logic_vector(6 downto 0);
  signal rightleds : std_logic_vector(6 downto 0);

  type rom_type is array(0 to 3) of std_logic_vector(7 downto 0);
  constant ROM : rom_type := (X"C5", X"EE", X"48", X"40");

  signal rom_data : std_logic_vector(7 downto 0);

begin

  -----------------------------------------------------------------------------
  --
  -- For lab 4, you will probably want to make changes starting from here.
  --

  -- High-frequency clock divider

  fast_clock_divider : process (clk)
  begin
    if clk'event and clk = '1' then  -- rising clock edge
      cnt24 <= cnt24 + 1;
    end if;
  end process;

  -- slow_clk <= cnt24(0);              -- faster version for simulation
  slow_clk <= cnt24(23);

  slow_counter : process (slow_clk)
  begin
    if slow_clk'event and slow_clk = '1' then              -- rising clock edge
      cnt8 <= cnt8 + 1;
    end if;
  end process;

  read_rom: process (clk)
    begin
    if clk'event and clk = '1' then
      rom_data <= ROM(conv_integer(cnt8(1 downto 0)));
    end if;
  end process;

  -- 
  -- End of suggested lab4 change region
  --  
  -----------------------------------------------------------------------------

  -- Two copies of the hex2led module, one for each 7-segment display

  l_h2l : hex2led
    port map (
      hex => rom_data(7 downto 4),
      led => leftleds
    );

  r_h2l : hex2led
    port map (
      hex => rom_data(3 downto 0),
      led => rightleds
    );

  -- Connect the decoder modules to the appropriate ports
  
  ( PB_D6, PB_D5, PB_D4, PB_D3, PB_D2, PB_D1, PB_D0 ) <= leftleds;
  
  ( PB_D14, PB_D13, PB_D12, PB_D11, PB_D10, PB_D9, PB_D8 ) <= rightleds;
  
  PB_D7 <= '0';
  PB_D15 <= '0';            -- decimal points

end imp;
